# Items
from __future__ import division

import data
from cocos.sprite import Sprite
from cocos.collision_model import AARectShape
from cocos.euclid import Vector2
from story import story
from character import Character
from pyglet import image, event
from ui import getUI
import main

class Item(event.EventDispatcher):
    def __init__(self, name, screen_index, position, sprite, visible=True):
        self.name = name
        self.screen_index = screen_index
        self.sprite = sprite
        self.glow = Sprite(image.load(data.filepath('itemGlow.png')))
        self.sprite.position = position
        self.visible = visible
        self.cshape = AARectShape(Vector2(*position), sprite.width / 2, sprite.height / 2)
    
    def playerNear(self):
        if not self.visible: return
        getUI().dialogBox(self, "<%s> to pick me up" % main.kb.action_str())
        self.sprite.add(self.glow, z=-1)
    
    def playerFar(self):
        if not self.visible: return
        getUI().removeDialogFromItem(self)
        self.sprite.remove(self.glow)
    
    def action(self, actor):
        # pick me up
        if not self.visible:
            return
        self.sprite.visible = self.visible = False
        actor.aquireItem(self)
        getUI().removeDialogFromItem(self)
        self.dispatch_event('on_aquire_item', self)
    
Item.register_event_type('on_aquire_item')        

class NPC(Item):
    def playerNear(self):
        pass
    
    def playerFar(self):
        pass

    def action(self, actor):
        pass

    def doneTalking(self):
        self._talking = False
        self.dispatch_event('on_finished_talking', self)

class Switch(Item):
    def __init__(self, name, screen_index, position):
        super(Switch, self).__init__(name, screen_index, position, Sprite(image.load(data.filepath('switch.png'))))
        self.on = False

    def playerNear(self):
        self.sprite.add(self.glow, z=-1)

    def playerFar(self):
        self.sprite.remove(self.glow)

    def action(self, actor):
        self.on = not self.on
        self.sprite.image = self.sprite.image.get_texture().get_transform(flip_x=True)
        self.dispatch_event('on_switch_switch', self)

Switch.register_event_type('on_switch_switch')    

class ChattyNPC(NPC):
    def __init__(self, name, screen_index, position, sprite, visible=True):
        super(ChattyNPC, self).__init__(name, screen_index, position, sprite, visible)
        self._talking = False
        self._help_text = True
        self.readyForAction = True
        
    def playerNear(self):
        if not self.visible or not self._help_text: return
        self._help_text = False
        getUI().dialogBox(self, "<%s> to talk" % main.kb.action_str())
    
    def playerFar(self):
        if not self.visible: return
        getUI().removeDialogFromItem(self)

    def action(self, actor):
        if not self._talking and self.readyForAction:
            self.playerFar()
            getUI().dialogList(self, main.dialogTree(self.name))
            self._talking = True

ChattyNPC.register_event_type('on_finished_talking')
