# Mission Framework

import pyglet
from story import story
from items import Item, NPC, ChattyNPC, Switch
import data
from character import Character, Direction, Walk, WalkTo, WazSpawn
from cocos.sprite import Sprite
from cocos.director import director
from ui import getUI, BlinkDialogEvent, addMessage, Message
from cocos.actions import sequence
from cocos.actions.instant_actions import CallFunc, Hide, Show
from cocos.actions.interval_actions import Delay, FadeOut, JumpTo, JumpBy, RotateBy
import main
import new

class Mission(pyglet.event.EventDispatcher):
    def __init__(self, name, descr):
        self._started = False
        self._completed = False
        self.name = name
        self.descr = descr
        self.notify_start = True
        self.next = ''
        self.hidden = False
        self._entities = [ [] for i in xrange(main.n_scenes) ] # indexed for each screen
            
    def canStart(self):
        return not self._started
    
    def isCompleted(self):
        return self._completed
    
    def start(self):
        print "Started " + self.name
        assert self.canStart(), "Cannot restart a completed mission"
        self._started = True
        if not self.hidden:
            self.dispatch_event('on_mission_start', self)
    
    def step(self, dt):
        pass
    
    def entities(self, screen_index):
        return self._entities[screen_index]

    def addEntity(self, entity):
        self._entities[entity.screen_index].append( entity )
        return entity
    
    def end(self):
        print "Finished " + self.name
        if not self.hidden:
            self.dispatch_event('on_mission_complete', self)
        
Mission.register_event_type('on_mission_complete')
Mission.register_event_type('on_mission_start')

def img(name):
    return pyglet.image.load(data.filepath(name))

class People(Mission):
    def __init__(self):
        imgs = [ 'Philip.png', 'hero.png', 'Henry.png', 'Franck.png', 'Clarissa.png' ]
        import random
        def genP(name, screen, location, klass=NPC):
            return klass(name, screen, location, Character(random.choice(imgs)))
        super(People, self).__init__('people', 'Random people on the map')
        self.hidden = True
        locations = (
            ( 0, 22, 15),
            ( 0, 16, 8),
            ( 0, 20, 3),
            ( 0, 5, 12),
            ( 0, 2, 5),
            ( 1, 6, 8),
            ( 1, 7, 6),
            ( 1, 17, 6),
            ( 1, 18, 8),
            ( 1, 7, 13),
            ( 1, 21, 5),
            ( 2, 4, 2),
            ( 2, 12, 3),
            ( 2, 20, 2),
            (2, 22, 11),
            (2,2,14),
            (3,22,9),
            (3,14,8),
            (3,2,2),
            (4,3,9),
            (4,12,5),
            (4,20,2),
            (5,3,5),
            (5,12,2),
            (5,20,10),
            (7,10,4),
            (7, 11,9),
            (7, 15,10),
            (8, 15,9),
            (8, 16,9),
            )
        names = [
            'JAMES',
            'JOHN',
            'ROBERT',
            'MICHAEL',
            'WILLIAM',
            'DAVID',
            'RICHARD',
            'CHARLES',
            'JOSEPH',
            'THOMAS',
            'CHRISTOPHER',
            'DANIEL',
            'PAUL',
            'MARK',
            'DONALD',
            'GEORGE',
            'MARY',
            'PATRICIA',
            'LINDA',
            'BARBARA',
            'ELIZABETH',
            'JENNIFER',
            'MARIA',
            'SUSAN',
            'MARGARET',
            'DOROTHY',
            'LISA',
            'NANCY',
            'KAREN',
            'BETTY',
            'HELEN',
            'SANDRA',
            'DONNA',
            ]
        for i in xrange(len(locations)):
            l = locations[i]
            if names[i] in ("JOHN",):
                k = ChattyNPC
            else:
                k = NPC
            story.registerNPC(self.addEntity(genP(names[i], l[0], ((l[1]*32)+16, (l[2]*32)+16), klass=k)))
        _loc = ( (32*9, 32*10), (32*15, 32*10), (32*9, 32*7), (32*15, 32*7) )
        for i in xrange(1, 5):
            story.registerItem(self.addEntity(Switch('switch%d' % i, 6, _loc[i-1])))
            
    def on_mission_start(self, mission):
        pass

    def on_mission_complete(self, mission):
        pass

class GetFruitMission(Mission):
    def __init__(self):
        story.registerNPC(NPC('evilmartin', 1, (300, 50), Character('martin.png')))
        apple = Item('apple', 0, (416, 250), Sprite(img('apple.png')))
        monkey = Item('monkey', 0, (416, 250), Sprite(img('monkey.png')), visible=False)
        story.registerItem(apple)
        story.registerItem(monkey)
        apple.push_handlers(self)
        self.schedule = []
        super(GetFruitMission, self).__init__('GetFruit', 'Find an APPLE')
        self.next = 'FindOutAboutCurse'
        self.notify_start = False
    
    def start(self):
        super(GetFruitMission, self).start()
        apple = story.item('apple')
        self._entities[apple.screen_index].append(apple)
        self.evil_martin = story.npc('evilmartin')
        self._entities[self.evil_martin.screen_index].append(self.evil_martin)
        self._db = getUI().dialogBox(self.evil_martin, "A Curse on You!")
        self._db.do(BlinkDialogEvent(0.8, 0.2))
        addMessage('Ignore Martin...', 2)
        addMessage('To walk use <%s> <%s> <%s> <%s>' % (
            main.kb.forward_str(),
            main.kb.backward_str(),
            main.kb.left_str(),
            main.kb.right_str()),
                   2, display_in=0.1)
        addMessage('Walk EAST to find that APPLE you have a hankering for', 2, display_in=0.1)                                                  
        return self
    
    def step(self, dt):
        pass
    
    def end(self):
        self._completed = True
        self._db.stop()
        john = story.npc('JOHN')
        john.name = 'JOHN1'
        story.updateNPC('JOHN', john)
        self.evil_martin.sprite.parent.parent.ui.removeDialogFromItem(self.evil_martin)
        self.evil_martin.sprite.visible = False
        super(GetFruitMission, self).end() # call to emit event

    def on_aquire_item(self, item):
        if item.name == 'apple':
            main.Waz._hero.removeItem(item)
            main.Waz._hero.aquireItem(story.item('monkey'))
            # need to transform the item
            self.end()

class FindOutAboutCurse(Mission):
    def __init__(self):
        story.registerNPC(ChattyNPC('kindclarissa', 1, (300, 100), Character('Clarissa.png')))
        self.schedule = []
        self._need_push_clarissa_msgs = True
        self.screen = -1
        super(FindOutAboutCurse, self).__init__('FindOutAboutCurse', 'Figure out why the apple became a monkey')
        self.next = 'TalkToFranck'
        self.notify_start = False
    
    def start(self):
        super(FindOutAboutCurse, self).start()
        main.Waz.push_handlers(self)
        self.kind_clarissa = story.npc('kindclarissa')
        self.kind_clarissa.push_handlers(self)
        self._entities[self.kind_clarissa.screen_index].append(self.kind_clarissa)
        addMessage('Way to go! That looks like a tasty apple.', 2, display_in=0)
        addMessage('Legolas\' Mother! The apple you put in your pocket has turned into a monkey!', 3, display_in=0.5)
        addMessage('Probably should go see Martin about that curse...', 1, display_in=0)
        return self
    
    def step(self, dt):
        if self._need_push_clarissa_msgs and self.screen == 1:
            addMessage('Looks like you missed Martin.', 1.5)
            addMessage('But who is that woman? Should probably talk to her', 2)
            self._need_push_clarissa_msgs = False
    
    def end(self):
        self._completed = True
        self.kind_clarissa.sprite.direction = Direction.West
        self.kind_clarissa.sprite.do(WalkTo((-10, 50), 4))
        super(FindOutAboutCurse, self).end() # call to emit event

    def on_switch_screen(self, index):
        self.screen = index

    def on_finished_talking(self, npc):
        if npc.name == 'kindclarissa':
            self.end()

class TalkToFranck(Mission):
    def __init__(self):
        story.registerNPC(ChattyNPC('wisefranck', 2, (80, 256), Character('Franck.png')))
        super(TalkToFranck, self).__init__('TalkToFranck', 'Find FRANCK')
        self.next = [ 'FindToiletBrush', 'FindBook'] #, 'FindCD', 'FindCandy', 'FindSoap']

    def start(self):
        super(TalkToFranck, self).start()
        main.Waz.push_handlers(self)
        self.wise_franck = story.npc('wisefranck')
        self.wise_franck.push_handlers(self)
        self._entities[self.wise_franck.screen_index].append(self.wise_franck)
        addMessage('Seems a little formulaic...', 2)
        addMessage('Oh, why not? Go see FRANCK', 2)
        return self

    def end(self):
        self._completed = True
        self.wise_franck.sprite.direction = Direction.West
        self.wise_franck.sprite.do(sequence(WalkTo((48, 256), 1), FadeOut(1)))
        super(TalkToFranck,self).end()

    def on_finished_talking(self, npc):
        if npc.name == 'wisefranck':
            self.end()

class FindToiletBrush(Mission):
    def __init__(self):
        story.registerNPC(NPC('philip', 8, (350, 400), Character('Philip.png')))
        story.registerNPC(ChattyNPC('henry', 8, (224,224), Character('Henry.png')))
        story.registerItem(Item('trashtoiletbrush', 8, (408, 446), Sprite(img('TBTrash.png')), visible=False))
        story.registerItem(Item('toiletbrush', 8, (350, 400), Sprite(img('toiletbrush.png'))))
        self._needsDrama = True
        self._inDrama = False
        self.screne = -1
        super(FindToiletBrush,self).__init__('FindToiletBrush', 'Find a recently used Toilet Brush')

    def start(self):
        super(FindToiletBrush, self).start()
        self.addEntity(story.item('trashtoiletbrush'))
        self.addEntity(story.item('toiletbrush'))
        self.henry = self.addEntity(story.npc('henry'))
        self.henry.readyForAction = False
        self.henry.push_handlers(self)
        self.philip = self.addEntity(story.npc('philip'))
        self.philip.push_handlers(self)
        self.john = story.npc('JOHN1')
        self.monkey = self.addEntity(story.item('monkey'))
        main.Waz.push_handlers(self)

    def step(self, dt):
        if self._needsDrama and self.screne == 8:
            self._needsDrama = False
            self._inDrama = True
            getUI().dialogList(self.philip, [
                Message('This TOILET BRUSH stinks.', 2),
                Message('Into the trash with you!', 1),
                Message('Ya!', 0.1)
                ])
    
    def on_finished_talking(self, actor):
        if actor.name == 'philip':
            story.item('toiletbrush').sprite.do(
                WazSpawn(JumpTo(position=(384,416), duration=1), RotateBy(540, 1)) +
                Hide())
            def showTTB(): story.item('trashtoiletbrush').visible = True
            self.philip.sprite.do(Delay(1) + CallFunc(lambda : showTTB()))
            db = getUI().dialogBox(self.henry, 'Philip! That\'s nasty! We\'ve got customers!')
            self.henry.sprite.do(Delay(2) + CallFunc(lambda : getUI().removeDialogFromItem(self.henry)))
            self.henry.readyForAction = True
        elif actor.name == 'henry':
            self.henry.name = 'henry2'
            story.updateNPC('henry', self.henry)
        elif actor.name == 'henry2':
            self.henry.name = 'henry3'
            story.updateNPC('henry2', self.henry)
            def poo(s, actor):
                s.playerFar()
                self.monkey.visible = True
                director.scene._refresh_entities()
                self.monkey.sprite.do(JumpBy((0,0), 25, 3, 2))
                getUI().dialogList(s, [
                    Message('No! I hate monkeys!', 1),
                    Message('Get me out of here! They throw poo!', 1)])
                s.sprite.do(Delay(3) + WalkTo((550, 700), duration=4))
                self.monkey.sprite.do(Delay(4) + JumpTo((550, 700), 25, 6, 4))
                self.on_monkey_poo(s)
            self.john.action = new.instancemethod( poo, self.john, ChattyNPC )
        elif actor.name == 'henry4':
            self.henry.name = 'henry5'
            story.updateNPC('henry4', self.henry)
            story.item('trashtoiletbrush').sprite.do(Hide())
            story.item('toiletbrush').sprite.do(
                Show() +
                WazSpawn(JumpTo(position=self.henry.sprite.position, duration=2), RotateBy(540, 2)))
            main.Waz._hero.aquireItem(story.item('toiletbrush'))
            self.end()
            
    def on_monkey_poo(self, victim):
        self.henry.name = 'henry4'
        story.updateNPC('henry3', self.henry)
        
    def end(self):
        self._completed = True
        super(FindToiletBrush, self).end()

    def on_switch_screen(self, index):
        self.screne = index

class BookOfPotions(Mission):
    def __init__(self):
        super(BookOfPotions, self).__init__('FindBook', 'Find the book of potions')
        story.registerItem(Item('book', 0, (416,250), Sprite(img('book.png'))))

    def start(self):
        super(BookOfPotions, self).start()
        self.book = self.addEntity(story.item('book'))
        self.book.push_handlers(self)
        self.switches = [ story.item('switch1'), story.item('switch2'), story.item('switch3'), story.item('switch4') ]
        for s in self.switches:
            s.push_handlers(self)
        self._book_done = False
        if self._check_switches():
            self._appearBook()

    def _check_switches(self):
        # correct order is on on off on
        return (self.switches[0].on) \
               and (self.switches[1].on) \
               and (not self.switches[2].on) \
               and (self.switches[3].on)

    def _appearBook(self):
        addMessage('CACHUNK! Well something happened outside and EAST.', 3)
        self._book_done = True
        self.book.visible = True
        self.book.sprite.visible = True

    def on_switch_switch(self, switch):
        if self._check_switches():
            self._appearBook()
        else:
            addMessage('Nothing...', 0.5)
        
    def end(self):
        self._completed = True
        if story.getMission('FindToiletBrush').isCompleted():
            main.Waz.finale()

    def on_aquire_item(self, item):
        if item.name == 'book':
            self.end()
        
def createMissions():
    story.registerMission(People())
    story.registerMission(GetFruitMission())
    story.registerMission(FindOutAboutCurse())
    story.registerMission(TalkToFranck())
    story.registerMission(FindToiletBrush())
    story.registerMission(BookOfPotions())

