# Story Manager

import main

class Story(object):
    def __init__(self):
        self._objects = {}
        self._object_names_by_screen = [ [] for i in xrange(main.n_scenes) ]
        self._npcs = {}
        self._npc_names_by_screen = [ [] for i in xrange(main.n_scenes) ]
        self._missions = {}
        self._active_missions = []
        
    def registerItem(self, item):
        self._objects[item.name] = item
        self._object_names_by_screen[item.screen_index].append(item)
    
    def item(self, name):
        return self._objects[name]
    
    def itemsOnScreen(self, screen_index):
        return [ i for i in self._object_names_by_screen[screen_index] if i.visible ]
    
    def registerNPC(self, npc):
        self._npcs[npc.name] = npc
        self._npc_names_by_screen[npc.screen_index].append(npc)
    
    def npc(self, name):
        return self._npcs[name]

    def updateNPC(self, old_name, npc):
        del self._npcs[old_name]
        self._npcs[npc.name] = npc
        
    def npcsOnScreen(self, screen_index):
        return self._npc_names_by_screen[screen_index]

    def npcDialogTree(self, name):
        return self._dialog_tree[name]
        
    def registerMission(self, mission):
        mission.push_handlers(self)
        self._missions[mission.name] = mission
    
    def listMissions(self):
        return self._missions.values()
    
    def getMission(self, mission_name):
        return self._missions[mission_name]

    def getActiveMissions(self):
        return self._active_missions

    def startMission(self, mission_name):
        m = self.getMission(mission_name)
        assert m.canStart(), "Can't start this mission"
        self._active_missions.append(m)
        return m.start()

    def step(self, dt):
        for i in self.getActiveMissions():
            i.step(dt)

    def on_mission_complete(self, mission):
        self._active_missions.remove(mission)
    
story = Story()
